@@ -1,11 +1,11 @@
 /* AstroPrincipal.m
  * Commander of the Astrology module
  *
- * Copyright (C) 2003-2004 by vhf interservice GmbH
+ * Copyright (C) 2003-2006 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  2003-03-29
- * modified: 2004-10-06
+ * modified: 2006-05-13
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -50,41 +50,29 @@
 
 @implementation AstroPrincipal
 
-/* 137.1234 -> 137 7'24"
- */
-NSString *stringFromDeg(float deg)
-{   int		d, m, s;
-    NSString	*string;
-
-    d = (int)deg;
-    m = (int)floor((deg-d) * 60.0);
-    s = (int)floor((deg-d-m/60.0) * 3600.0 + 0.5);
-    string = [NSString stringWithFormat:DEGREE_FORMAT,
-                       [[NSString stringWithFormat:@"%d", d] stringWithLength:-4],
-                       [[NSString stringWithFormat:@"%d", m] stringWithLength:-2],
-                       [[NSString stringWithFormat:@"%d", s] stringWithLength:-2] ];
-    return string;
-}
-
 /*
  * created:  1999-03-15
- * modified: 2004-07-31
+ * modified: 2006-04-27 (call registration in preferences module)
  * register the defaults
  */
 + (void)initialize
-{   NSMutableDictionary	*registrationDict = [NSMutableDictionary dictionary];
+{
+    //[AstroController registerDefaults];	// not loaded yet
+    NSMutableDictionary	*registrationDict = [NSMutableDictionary dictionary];
     NSArray		*objects;
 
     [registrationDict setObject:vhfStringFromRGBColor([NSColor whiteColor]) forKey:@"astroBGColor"];
+    [registrationDict setObject:@"YES" forKey:@"astroShowPanel"];
     [registrationDict setObject:@"YES" forKey:@"astroShowRetrograde"];
     [registrationDict setObject:@"%d.%m.%Y" forKey:@"astroDateFormat"];
     [registrationDict setObject:@"vhfAstro" forKey:@"astroFont"];
     //[registrationDict setObject:@"YES" forKey:@"astroShowHouseSize"];
     //[registrationDict setObject:@"YES" forKey:@"astroShowHouseMaxima"];
     [registrationDict setObject:@"YES" forKey:@"astroTopocentricPosition"];
-    //[registrationDict setObject:@"YES" forKey:@"astroShowWaves"];
-    //[registrationDict setObject:@"YES" forKey:@"astroShowDeclination"];
-    //[registrationDict setObject:@"YES" forKey:@"astroACFix"];
+    [registrationDict setObject:@"NO"  forKey:@"astroShowDeclination"];
+    [registrationDict setObject:@"NO"  forKey:@"astroShowAspectGeo"];
+    [registrationDict setObject:@"NO"  forKey:@"astroACFix"];
+    [registrationDict setObject:@"0"   forKey:@"astroInterval"];
     objects = [NSArray arrayWithObjects:@"Sun", @"Moon", @"Mercury", @"Venus", @"Mars", @"Jupiter", @"Saturn", @"Uranus", @"Neptune", @"Pluto", @"Chiron", @"True Node", @"True Apogee", nil];
     [registrationDict setObject:objects forKey:@"astroObjects"];
 
@@ -101,6 +89,30 @@ NSString *stringFromDeg(float deg)
     return instance;
 }
 
+/*  137.1234 ->  137 7'24"
+ * -137.1234 -> -137 7'24"
+ */
+NSString *stringFromDeg(float deg)
+{
+    return [AstroPrincipal stringFromDeg:deg];
+}
++ (NSString*)stringFromDeg:(float)deg
+{   int		d, m, s;
+    NSString	*string;
+
+    //if (deg < 0)
+    //    NSLog(@"stringFromDeg(): does not properly support negative degrees !");
+    d = (int)deg;
+    m = (int)floor((deg-d) * 60.0);
+    s = (int)floor((deg-d-m/60.0) * 3600.0 + 0.5);
+    string = [NSString stringWithFormat:DEGREE_FORMAT,
+                       [[NSString stringWithFormat:@"%d", d] stringWithLength:(deg < 0.0) ? -4 : -3],
+                       [[NSString stringWithFormat:@"%d", m] stringWithLength:-2],
+                       [[NSString stringWithFormat:@"%d", s] stringWithLength:-2] ];
+    return string;
+}
+
+
 /* init
  */
 - init
@@ -134,10 +146,12 @@ NSString *stringFromDeg(float deg)
                         action:@selector(showAstroPanel:)
                  keyEquivalent:@""];
     [[[toolMenu itemArray] objectAtIndex:[[toolMenu itemArray] count]-1] setTarget:self];
+    if (Prefs_ShowAstroPanel)
+        [self showAstroPanel:self];
 
     /* create event manager, city manager, time zone manager */
-    eventManager = [EventManager new];
-    cityManager = [CityManager new];
+    eventManager    = [EventManager new];
+    cityManager     = [CityManager new];
     timeZoneManager = [TimeZoneManager new];
 
     return self;
@@ -242,7 +256,7 @@ NSString *stringFromDeg(float deg)
         AstroChart	*astroChart;
 
         for ( i=[[NSApp windows] count]-1; i>=0; i-- )
-        {   doc = [NSApp documentInWindow:[[(App*)NSApp windows] objectAtIndex:i]];
+        {   doc = [(App*)NSApp documentInWindow:[[(App*)NSApp windows] objectAtIndex:i]];
             if ( [[doc name] hasPrefix:CHART_PREFIX] )
                 break;
         }
@@ -266,25 +280,28 @@ NSString *stringFromDeg(float deg)
 }
 
 /* notification that the date was set, and we have to load the corresponding map image
- *
  * object   NSCalendarDate with date and time of map to load
+ * modified: 2006-04-03
  */
 - (void)updateMapForDate:(NSNotification*)notification
 {   NSCalendarDate	*date = [notification object];
 
     if ([date isKindOfClass:[NSCalendarDate class]])
     {   int		i;
-        Document	*doc;
+        Document	*doc = [(App*)NSApp currentDocument];
+        DocView		*view = [doc documentView];
         Map		*map;
-        DocView		*view;
 
-        for ( i=[[NSApp windows] count]-1; i>=0; i-- )
-        {   doc = [NSApp documentInWindow:[[(App*)NSApp windows] objectAtIndex:i]];
-            if ( [[doc name] hasPrefix:MAP_PREFIX] )
-                break;
+        if ( ![[doc name] hasPrefix:MAP_PREFIX] )
+        {
+            for ( i=[[NSApp windows] count]-1; i>=0; i-- )
+            {   doc = [(App*)NSApp documentInWindow:[[(App*)NSApp windows] objectAtIndex:i]];
+                if ( [[doc name] hasPrefix:MAP_PREFIX] )
+                    break;
+            }
+            if ( i < 0 )
+                return;
         }
-        if ( i < 0 )
-            return;
 
         view = [doc documentView];
         map = [[view statusDict] objectForKey:@"map"];
