@@ -1,11 +1,11 @@
 /* CityManager.m
  * controller class and files owner of the CityPanel
  *
- * Copyright (C) 2003-2004 by vhf interservice GmbH
+ * Copyright (C) 2003-2005 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  2003-06-29
- * modified: 2004-10-06
+ * modified: 2005-11-16 (use MODULENAME makro)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -31,6 +31,7 @@
 #include "../Cenon/App.h"
 #include "../Cenon/functions.h"
 #include "../Cenon/messages.h"
+#include "astroLocations.h"
 #include "astroMessages.h"
 #include "AstroPrincipal.h"
 #include "Astro.bproj/AstroController.h"
@@ -358,7 +359,7 @@
 {   NSString		*path, *dir;
     NSFileManager	*fileManager = [NSFileManager defaultManager];
 
-    dir = [userLibrary() stringByAppendingPathComponent:@"Astro"];
+    dir = [userLibrary() stringByAppendingPathComponent:MODULENAME];
     if (![fileManager fileExistsAtPath:dir])	// create Astro folder
         [fileManager createDirectoryAtPath:dir attributes:nil];
     dir = [dir stringByAppendingPathComponent:CITIES_FOLDER];
