@@ -1,11 +1,11 @@
 /* EventManager.m
  * controller class and files owner of the EventPanel
  *
- * Copyright (C) 2003-2005 by vhf interservice GmbH
+ * Copyright (C) 2003-2006 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  2003-06-22
- * modified: 2005-08-31 (sub-categories, Cenon data archives added)
+ * modified: 2006-01-15 (date formats)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -31,9 +31,11 @@
 #include "../Cenon/App.h"
 #include "../Cenon/functions.h"
 #include "../Cenon/messages.h"
+#include "astroLocations.h"
 #include "astroMessages.h"
 #include "AstroPrincipal.h"
 #include "Astro.bproj/AstroController.h"
+#include "AstroDate.h"	// date checks etc.
 #include "AAFImport.h"
 #include "CityManager.h"
 #include "TimeZoneManager.h"
@@ -210,13 +212,16 @@ static int indexFromType(NSString *typeString)
     [[Astro_Principal timeZoneManager] showPanel:self];
 }
 
+/* add new entry
+ * modified: 2006-01-15
+ */
 - (void)add:sender
 {   NSMutableDictionary	*event = [NSMutableDictionary dictionary];
     int			folderIx = [eventBrowser selectedRowInColumn:0], eventIx, i;
     NSMutableArray	*events;
-    NSString		*dateFormat_Z = [NSString stringWithFormat:@"%@ %%H:%%M %%Z", Prefs_DateFormat];
-    NSString		*dateFormat_z = [NSString stringWithFormat:@"%@ %%H:%%M %%z", Prefs_DateFormat];
-    NSString		*errorDate = ([Prefs_DateFormat isEqual:@"%d.%m.%Y"]) ? DATE1_STRING : DATE2_STRING;
+    NSString		*dateFormat = astroDateFormatForString([dateField stringValue]);
+    NSString		*dateFormat_Z = [dateFormat stringByAppendingString:@" %H:%M %Z"];
+    NSString		*dateFormat_z = [dateFormat stringByAppendingString:@" %H:%M %z"];
     NSString		*string;
 
     if (folderIx < 0)
@@ -238,11 +243,23 @@ static int indexFromType(NSString *typeString)
         }
     }
 
-    /* add data fields */
-    if (![eventField stringValue])
+    if (![eventField stringValue])	// empty title
         return;
+
+    /* name already exists -> clear fields (?) */
+    for (i=0; i<[events count]; i++)
+    {   NSString	*name = [[events objectAtIndex:i] objectForKey:@"name"];
+
+        if ( [[eventField stringValue] isEqual:name] )
+        {
+            [self clear:self];
+            return;
+        }
+    }
+
+    /* add data fields */
     [event setObject:[eventField stringValue] forKey:@"name"];
-    if ([[dateField stringValue] length] == 10 && [[timeField stringValue] length] == 5)
+    if ( dateFormat && [[timeField stringValue] length] == 5)
     {   NSString	*tzStr = [tzField stringValue];
         NSString	*string = [NSString stringWithFormat:@"%@ %@ %@",
                                             [dateField stringValue], [timeField stringValue], tzStr];
@@ -254,7 +271,7 @@ static int indexFromType(NSString *typeString)
             date = [NSCalendarDate dateWithString:string calendarFormat:dateFormat_Z];
 
         if (!date)
-        {   NSRunAlertPanel(@"", WRONGDATEFORMAT_STRING, OK_STRING, nil, nil, errorDate);
+        {   NSRunAlertPanel(@"", WRONGDATEFORMAT_STRING, OK_STRING, nil, nil, astroErrorDate());
             return;
         }
 
@@ -265,7 +282,7 @@ static int indexFromType(NSString *typeString)
         [event setObject:string forKey:@"date"];
     }
     else
-    {   NSRunAlertPanel(@"", WRONGDATEFORMAT_STRING, OK_STRING, nil, nil, errorDate);
+    {   NSRunAlertPanel(@"", WRONGDATEFORMAT_STRING, OK_STRING, nil, nil, astroErrorDate());
         return;
     }
     if ((string = typeFromIndex([typePopup indexOfSelectedItem])))
@@ -303,9 +320,9 @@ static int indexFromType(NSString *typeString)
     int			eventIx  = [eventBrowser selectedRowInColumn:1], i;
     NSMutableArray	*events;
     NSMutableDictionary	*event;
-    NSString		*dateFormat_Z = [NSString stringWithFormat:@"%@ %%H:%%M %%Z", Prefs_DateFormat];
-    NSString		*dateFormat_z = [NSString stringWithFormat:@"%@ %%H:%%M %%z", Prefs_DateFormat];
-    NSString		*errorDate = ([Prefs_DateFormat isEqual:@"%d.%m.%Y"]) ? DATE1_STRING : DATE2_STRING;
+    NSString		*dateFormat = astroDateFormatForString([dateField stringValue]);
+    NSString		*dateFormat_Z = [dateFormat stringByAppendingString:@" %H:%M %Z"];
+    NSString		*dateFormat_z = [dateFormat stringByAppendingString:@" %H:%M %z"];
     NSString		*string;
 
     if (eventIx < 0)
@@ -340,7 +357,7 @@ static int indexFromType(NSString *typeString)
         return;
     }
 
-    if ([[dateField stringValue] length] == 10 && [[timeField stringValue] length] == 5)
+    if ( dateFormat && [[timeField stringValue] length] == 5 )
     {   NSString	*tzStr = [tzField stringValue];
         NSString	*string = [NSString stringWithFormat:@"%@ %@ %@",
                                             [dateField stringValue], [timeField stringValue], tzStr];
@@ -352,7 +369,7 @@ static int indexFromType(NSString *typeString)
             date = [NSCalendarDate dateWithString:string calendarFormat:dateFormat_Z];
 
         if (!date)
-        {   NSRunAlertPanel(@"", WRONGDATEFORMAT_STRING, OK_STRING, nil, nil, errorDate);
+        {   NSRunAlertPanel(@"", WRONGDATEFORMAT_STRING, OK_STRING, nil, nil, astroErrorDate());
             return;
         }
 
@@ -365,7 +382,7 @@ static int indexFromType(NSString *typeString)
     else if (![[dateField stringValue] length])	// this uses the current date
         [event removeObjectForKey:@"date"];
     else
-    {   NSRunAlertPanel(@"", WRONGDATEFORMAT_STRING, OK_STRING, nil, nil, errorDate);
+    {   NSRunAlertPanel(@"", WRONGDATEFORMAT_STRING, OK_STRING, nil, nil, astroErrorDate());
         return;
     }
 
@@ -586,9 +603,9 @@ static int indexFromType(NSString *typeString)
     [eventBrowser setTarget:self];
     [eventBrowser setAction:@selector(doClick:)];
 
-    path = [NSString stringWithFormat:@"%@/Astro/event.db", userLibrary()];
+    path = vhfPathWithPathComponents(userLibrary(), MODULENAME, @"event.db", nil);
     if ( ![fileManager fileExistsAtPath:path] )
-        path = [NSString stringWithFormat:@"%@/Astro/event.db", localLibrary()];
+        path = vhfPathWithPathComponents(localLibrary(), MODULENAME, @"event.db", nil);
 
     [eventDict release];
     [folders release];
@@ -599,9 +616,9 @@ static int indexFromType(NSString *typeString)
     for (i=0; i<2; i++)
     {
         if (!i)
-            path = [NSString stringWithFormat:@"%@/Astro/Data", localLibrary()];
+            path = vhfPathWithPathComponents(localLibrary(), MODULENAME, @"Data", nil);
         else
-            path = [NSString stringWithFormat:@"%@/Astro/Data", userLibrary()];
+            path = vhfPathWithPathComponents(userLibrary(), MODULENAME, @"Data", nil);
         array = [fileManager directoryContentsAtPath:path];
         for (j=0; j<(int)[array count]; j++)
         {   NSString	*file     = [array objectAtIndex:j];
@@ -629,7 +646,7 @@ static int indexFromType(NSString *typeString)
 - (void)saveEvents
 {   NSString	*path;
 
-    path = [NSString stringWithFormat:@"%@/Astro/event.db", userLibrary()];
+    path = vhfPathWithPathComponents(userLibrary(), MODULENAME, @"event.db", nil);
     [eventDict writeToFile:path atomically:YES];
 }
 
@@ -808,37 +825,32 @@ static int indexFromType(NSString *typeString)
 {   NSTimeZone	*tz = [notification object];
 
     if ([tz isKindOfClass:[NSTimeZone class]])
-    {   NSString	*errorDate = ([Prefs_DateFormat isEqual:@"%d.%m.%Y"]) ? DATE1_STRING : DATE2_STRING;
-        NSString	*dateStr = [dateField stringValue];
+    {   NSString	*dateStr = [dateField stringValue];
+        NSString	*dateFormat = astroDateFormatForString([dateField stringValue]);
         NSString	*timeStr = [timeField stringValue];
         int		y, m, d, H, M;
         NSCalendarDate	*date;
 
+        if (!dateFormat)
+        {   NSRunAlertPanel(@"", WRONGDATEFORMAT_STRING, OK_STRING, nil, nil, astroErrorDate());
+            return;
+        }
         if ([timeStr length] != 5)
         {   NSRunAlertPanel(@"", WRONGDATEFORMAT_STRING, OK_STRING, nil, nil, @"HH:MM");
             return;
         }
-        if ([dateStr length] != 10)
-        {   NSRunAlertPanel(@"", WRONGDATEFORMAT_STRING, OK_STRING, nil, nil, errorDate);
+
+        if (![[AstroDate sharedInstance] getYear:&y month:&m day:&d fromDateString:dateStr])
+        {   NSRunAlertPanel(@"", WRONGDATEFORMAT_STRING, OK_STRING, nil, nil, astroErrorDate());
             return;
         }
 
-        if ([Prefs_DateFormat isEqual:@"%d.%m.%Y"])
-        {   d = [[dateStr substringWithRange:NSMakeRange(0, 2)] intValue];
-            m = [[dateStr substringWithRange:NSMakeRange(3, 2)] intValue];
-            y = [[dateStr substringWithRange:NSMakeRange(6, 4)] intValue];
-        }
-        else
-        {   y = [[dateStr substringWithRange:NSMakeRange(0, 4)] intValue];
-            m = [[dateStr substringWithRange:NSMakeRange(5, 2)] intValue];
-            d = [[dateStr substringWithRange:NSMakeRange(8, 2)] intValue];
-        }
         H = [[timeStr substringToIndex:  2] intValue];
         M = [[timeStr substringFromIndex:3] intValue];
 
         date = [NSCalendarDate dateWithYear:y month:m day:d hour:H minute:M second:0 timeZone:tz];
         if (!date)
-        {   NSRunAlertPanel(@"", WRONGDATEFORMAT_STRING, OK_STRING, nil, nil, errorDate);
+        {   NSRunAlertPanel(@"", WRONGDATEFORMAT_STRING, OK_STRING, nil, nil, astroErrorDate());
             return;
         }
 
