@@ -2,11 +2,11 @@
  * Commander of the map
  * This class is associated to a view containing the map.
  *
- * Copyright (C) 2003-2004 by vhf interservice GmbH
+ * Copyright (C) 2003-2006 by vhf interservice GmbH
  * Author:   Georg Fleischmann, Ilonka Fleischmann
  *
  * created:  2003-07-14
- * modified: 2004-03-19
+ * modified: 2006-05-31 (house lines)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -35,12 +35,15 @@
 #include "../Cenon/functions.h"
 #include "../Cenon/messages.h"
 #include "../Cenon/propertyList.h"
-#include "AstroPrincipal.h"	// shared ephemeris object
+#include "VTextAstroAdditions.h"	// convenient text methods
+#include "AstroPrincipal.h"		// shared ephemeris object
+#include "EphemerisSearch.h"
+#include "TopocentricCycle.h"
 #include "astroCommon.h"
 #include "Astro.bproj/AstroController.h"
 #include "Map.h"
 
-#define	EarthRadiusA	6378137.0
+#define	EarthRadiusA	6378137.0	// equator radius
 #define	EarthRadiusB	6356752.0
 #define Eccentricity	(EarthRadiusA - EarthRadiusB)/EarthRadiusA
 #define	OneDegInM	(2.0*Pi*EarthRadiusA / 360.0)	// meter -> degree (2*Pi*r/360 -> 1 degree in meter)
@@ -94,6 +97,7 @@ static CIAMap	*ciaMap = nil;	// one shared cia map for all views
  * x, y are the screen coordinates in point. the latitude/longitude entries are in degrees.
  *   "longitude"   = { latitude=(x,y); latitude=(x,y); ... };
  * ...
+ * modified: 2006-04-05 (projection + deg0 for CIA maps too)
  */
 - (id)initWithContentsOfDictionary:(NSDictionary*)dict
 {
@@ -102,7 +106,7 @@ static CIAMap	*ciaMap = nil;	// one shared cia map for all views
     mapDict = [dict mutableCopy];
 
     /* CIA map */
-    if ( ![dict objectForKey:@"projection"] )
+    if ( ![dict objectForKey:@"projection"] || [dict objectForKey:@"mapset"] )
     {
         mapset = [[dict objectForKey:@"mapset"] retain];
         center = NSMakePoint([dict floatForKey:@"centerLon"], [dict floatForKey:@"centerLat"]);
@@ -122,14 +126,33 @@ static CIAMap	*ciaMap = nil;	// one shared cia map for all views
             origin = NSMakePoint([dict floatForKey:@"x"], [dict floatForKey:@"y"]);
         if ([dict objectForKey:@"scale"])
             scale = [dict doubleForKey:@"scale"];
-        if ([dict objectForKey:@"deg0"])
-            deg0 = [dict floatForKey:@"deg0"];
         if ([dict objectForKey:@"grid"])
             freeGrid = [[dict objectForKey:@"grid"] retain];
         if ([dict objectForKey:@"bounds"])
             bounds = rectFromPropertyList([dict objectForKey:@"bounds"]);
-        projectionType = [[dict stringForKey:@"projection"] characterAtIndex:0];
     }
+    projectionType = [[dict stringForKey:@"projection"] characterAtIndex:0];
+    if ([dict objectForKey:@"deg0"])
+        deg0 = [dict floatForKey:@"deg0"];
+
+    colors = [[NSDictionary dictionaryWithObjectsAndKeys:
+              [NSColor blackColor],     @"Sun",
+              [NSColor darkGrayColor],  @"Moon",
+              [NSColor grayColor],      @"Node",
+              [NSColor grayColor],      @"True Node",
+              [NSColor lightGrayColor], @"Apogee",
+              [NSColor lightGrayColor], @"True Apogee",
+              [NSColor orangeColor],    @"Mercury",
+              [NSColor greenColor],     @"Venus",
+              [NSColor redColor],       @"Mars",
+              [NSColor cyanColor],      @"Jupiter",
+              [NSColor blueColor],      @"Saturn",
+              [NSColor purpleColor],    @"Uranus",
+              [NSColor magentaColor],   @"Neptune",
+              [NSColor brownColor],     @"Pluto",
+              [NSColor colorWithCalibratedRed:0.5 green:0.0 blue:1.0 alpha:1.0], @"Chiron",
+              [NSColor redColor],       @"AC",
+              [NSColor blueColor],      @"MC", nil] retain];
 
     return self;
 }
@@ -140,6 +163,11 @@ static CIAMap	*ciaMap = nil;	// one shared cia map for all views
     bounds = rect;
 }
 
+- (NSDictionary*)colors
+{
+    return colors;
+}
+
 NSPoint pointOnLineDistanceFromEnd(float dist, NSPoint p0, NSPoint p1)
 {   NSPoint	p, d;
     float	len;
@@ -154,24 +182,6 @@ NSPoint pointOnLineDistanceFromEnd(float dist, NSPoint p0, NSPoint p1)
     return p;
 }
 
-/* text convenience method
- */
-- (VText*)text:(NSString*)string
-        center:(NSPoint)p
-         color:(NSColor*)color
-          size:(float)textSize
-      fontName:(NSString*)fontName
-{   id		font = [NSFont fontWithName:fontName size:textSize];
-    VText	*text = [VText textGraphic];
-    NSRect	r;
-
-    [text setString:string font:font lineHeight:0.0 color:color];
-    r = [text bounds];
-    p.x -= r.size.width  / 2.0;
-    p.y -= r.size.height / 2.0;
-    [text moveTo:p];
-    return text;
-}
 
 /* map AC, DC, MC, IC at given date
  * we draw lines through locations where the axes are on a planet
@@ -188,10 +198,12 @@ NSPoint pointOnLineDistanceFromEnd(float dist, NSPoint p0, NSPoint p1)
     NSColor		*color;
     float		lineWidth = 1.0;
     BOOL		isACLine, wasACLine = NO;
-    NSString		*fontName = Prefs_AstroFont;
+    double		jd = vhfJulianDay(utc);	// julian day (reference -4712)
+    NSString		*astroFontName = (![(NSString*)Prefs_AstroFont length]) ? @"vhfAstro" : Prefs_AstroFont;
+    NSFont		*symbolFont14 = [NSFont fontWithName:astroFontName size:14];
 
-    [date release];
-    date = [utc retain];
+    [dateAxes release];
+    dateAxes = [utc retain];
 
     /* calculate all map positions with AC/IC/MC/DC on planets.
      * 1. get planet positions
@@ -206,7 +218,10 @@ NSPoint pointOnLineDistanceFromEnd(float dist, NSPoint p0, NSPoint p1)
         float		deg = [degreeDict floatForKey:planet], lon;
         NSPoint		p, pts[2], pa0, pa1, pb0, pb1, p0, p1;
 
-        lon = longitudeForMC(deg, utc);
+        lon = vhfLonForMC(deg, utc);
+        //lon1 = longitudeForMC(deg, utc);
+        //if (Diff(lon, lon1) > 0.0001)
+        //    printf("vhfLonForMC(): lon = %f = %f\n", lon, lon1);
         switch (projectionType)
         {
             case 'S':	// stereo:	straight lines from origin to bounds
@@ -258,8 +273,8 @@ NSPoint pointOnLineDistanceFromEnd(float dist, NSPoint p0, NSPoint p1)
             p = pointOnLineDistanceFromEnd(20.0, p0, p1); // line end (p*1) minus some space
             [groupMCText addObject:arc = [VArc arcWithCenter:p radius:8.0 filled:YES]];
             [arc setFillColor:[NSColor yellowColor]];
-            [groupMCText addObject:[self text:planetSymbol(planet) center:p
-                                        color:color size:14.0 fontName:fontName]];
+            [groupMCText addObject:[VText text:planetSymbol(planet) center:p
+                                         color:color font:symbolFont14]];
         }
     }
 
@@ -280,7 +295,10 @@ NSPoint pointOnLineDistanceFromEnd(float dist, NSPoint p0, NSPoint p1)
             lonStart = [self latLonForPointOnMap:viewBounds.origin].x;
         for ( lon = lonStart; lon <= lonStart+360.0; lon += jump)	// segment every 5 deg longitude
         {
-            lat = latitudeForAC(deg, sideralTime(utc, lon));
+            lat = vhfLatForAC(deg, vhfSideralTime(jd, lon));
+            //lat1 = latitudeForAC(deg, sideralTime(utc, lon));	// compare with old function
+            //if (Diff(lat, lat1) > 0.001)
+            //    printf("vhfLatForAC(): lat = %f = %f\n", lat, lat1);
             p1 = [self pointOnMapForLat:lat lon:lon];
 
 //NSLog(@"1 lat=%f lon=%f -> p1={%.2f %.2f}", lat, lon, p1.x, p1.y);
@@ -295,10 +313,13 @@ NSPoint pointOnLineDistanceFromEnd(float dist, NSPoint p0, NSPoint p1)
                  */
                 lStep = Max(Min((lon - l0) / (dist / 10.0), jump), 0.05);	// stepsize between min and max
                 for (l1=l0+lStep; l1<=lon; l1 += lStep)
-                {   NSCalendarDate	*st = sideralTime(utc, l1);
+                {   double	st = vhfSideralTime(jd, l1);
 
-                    lat = latitudeForAC(deg, st);
-                    isACLine = ( isAC(lat, deg, st) ) ? YES : NO;
+                    lat = vhfLatForAC(deg, st);
+                    isACLine = ( vhfIsAC(lat, deg, st) ) ? YES : NO;
+                    //lat1 = latitudeForAC(deg, sideralTime(utc, l1));	// compare with old function
+                    //if (Diff(lat, lat1) > 0.001)
+                    //    printf("vhfLatForAC(): lat = %f = %f\n", lat, lat1);
                     /* find lat close to equator for text */
                     if (projectionType != 'S')
                     {
@@ -379,8 +400,8 @@ NSPoint pointOnLineDistanceFromEnd(float dist, NSPoint p0, NSPoint p1)
                     continue;
                 [groupACText addObject:arc = [VArc arcWithCenter:ps[j] radius:8.0 filled:YES]];
                 [arc setFillColor:[NSColor yellowColor]];
-                [groupACText addObject:[self text:planetSymbol(planet) center:ps[j]
-                                            color:[NSColor redColor] size:14.0 fontName:fontName]];
+                [groupACText addObject:[VText text:planetSymbol(planet) center:ps[j]
+                                             color:[NSColor redColor] font:symbolFont14]];
             }
         }
     }
@@ -404,7 +425,430 @@ NSLog(@"1: lat=%f l1=%f -> p={%.2f %.2f}", lat, lon, p.x, p.y);
  */
 - (NSCalendarDate*)date
 {
-    return date;
+    return dateAxes;
+}
+
+/* map house tips at given date
+ * we draw lines through locations where the houses tips stay the same
+ * created:  2006-05-31
+ * modified: 2006-06-01
+ */
+- (VGroup*)mapHousesAtDate:(NSCalendarDate*)utc lat:(float)refLat lon:(float)refLon
+                     title:(NSString*)title inBounds:(NSRect)viewBounds
+{   VGroup		*group = [VGroup group];
+    VGroup		*groupACText = [VGroup group], *groupMCText = [VGroup group];
+    VLine		*line;
+    VArc		*arc;
+    int			i, j, houseCnt = 12, n;
+    float		lineWidth = 1.0;
+    double		jd = vhfJulianDay(utc);	// julian day (reference -4712)
+    NSString		*astroFontName = (![(NSString*)Prefs_AstroFont length]) ? @"vhfAstro" : Prefs_AstroFont;
+    NSFont		*symbolFont14 = [NSFont fontWithName:astroFontName size:14];
+    NSFont		*systemFont14 = [NSFont systemFontOfSize:14];
+    TopocentricCycle	*topo = [TopocentricCycle topocentricCycleAtDate:utc latitude:refLat longitude:refLon];
+    double		*houses = [topo houses:houseCnt];
+    NSArray		*houseColors;
+
+    houseColors = [NSArray arrayWithObjects:
+            [NSColor colorWithCalibratedHue:0.000 saturation:1.0 brightness:1.0 alpha:1.0], // aries
+            [NSColor colorWithCalibratedHue:0.028 saturation:1.0 brightness:1.0 alpha:1.0], // taurus
+            [NSColor colorWithCalibratedHue:0.056 saturation:1.0 brightness:1.0 alpha:1.0], // gemini
+            [NSColor colorWithCalibratedHue:0.083 saturation:1.0 brightness:1.0 alpha:1.0], // cancer
+            [NSColor colorWithCalibratedHue:0.167 saturation:1.0 brightness:1.0 alpha:1.0], // leo
+            [NSColor colorWithCalibratedHue:0.250 saturation:1.0 brightness:1.0 alpha:1.0], // virgo
+            [NSColor colorWithCalibratedHue:0.333 saturation:1.0 brightness:1.0 alpha:1.0], // libra
+            [NSColor colorWithCalibratedHue:0.500 saturation:1.0 brightness:1.0 alpha:1.0], // scorpio
+            [NSColor colorWithCalibratedHue:0.583 saturation:1.0 brightness:1.0 alpha:1.0], // sagittarius
+            [NSColor colorWithCalibratedHue:0.667 saturation:1.0 brightness:1.0 alpha:1.0], // capricorn
+            [NSColor colorWithCalibratedHue:0.750 saturation:1.0 brightness:1.0 alpha:1.0], // aquarius
+            [NSColor colorWithCalibratedHue:0.916 saturation:1.0 brightness:1.0 alpha:1.0], // pisces
+            nil];
+
+    //[dateHouses release];
+    //dateHouses = [utc retain];
+
+    /* calculate all map positions with AC/IC/MC/DC on planets.
+     * 1. get house positions
+     * 2. calculate house positions at lat/lon
+     */
+
+//i=0;
+   for (i=0; i<houseCnt/2; i++)
+    {   double		eDeg = houses[i], houseDeg = (i*360.0/houseCnt), tDeg;
+        float		lat, lon, lonStart = 0.0;
+        float		jump = (projectionType == 'F') ? 0.5 : 1.0;
+        NSPoint		p0 = NSMakePoint(-LARGE_COORD, -LARGE_COORD), p0_mir, p1, p1_mir;
+        VPolyLine	*polyLine[2] = {nil, nil};
+        NSPoint		textLatLon[2] = {{180.0, 180.0}, {180.0, 180.0}};
+        int		groupCount = [[group list] count];	// we need to know which polyLines were added
+        BOOL		isMCIC = (houseDeg == 90.0 || houseDeg == 270.0) ? YES : NO;
+        float		lonMCIC = (isMCIC) ? vhfLonForMC(houseDeg, utc) : 0.0;
+        NSColor		*color = [houseColors objectAtIndex:i];
+
+        /* line for IC, MC */
+        if (isMCIC)
+        {   NSPoint	p, pts[2], pa0, pa1, pb0, pb1;
+
+            lon = vhfLonForMC(eDeg, utc);
+            switch (projectionType)
+            {
+                case 'S':	// stereo:	straight lines from origin to bounds
+                    p = [self pointOnMapForLat:0.0 lon:lon];
+                    if (vhfIntersectVectorAndRect(p, NSMakePoint(origin.x-p.x, origin.y-p.y), viewBounds, pts) < 2)
+                    {   NSLog(@"mapChartWithDate: two intersections expected!");
+                        return nil;
+                    }
+                    p0 = pa0 = pb0 = origin;
+                    pa1 = ((origin.x-p.x) * (origin.x-pts[0].x) > 0.0) ? pts[0] : pts[1];
+                    pb1 = ((origin.x-p.x) * (origin.x-pts[0].x) > 0.0) ? pts[1] : pts[0];
+                    break;
+                case 'M':	// mercator:	straight lines from top to bottom
+                    pa0.x = pa1.x = [self pointOnMapForLat:0.0 lon:lon].x;
+                    pb0.x = pb1.x = [self pointOnMapForLat:0.0 lon:lon+180.0].x;
+                    pa0.y = pb0.y = viewBounds.origin.y;
+                    pa1.y = pb1.y = viewBounds.origin.y + viewBounds.size.height;
+                    break;
+                case 'F':	// free style
+                {   float	minLat = [self freeLatitudeLimitForLongitude:lon searchMinLatitude:YES];
+                    float	maxLat = [self freeLatitudeLimitForLongitude:lon searchMinLatitude:NO];
+
+                    /* we have to create the MC/IC lines at points within the grid */
+                    pa0 = [self pointOnMapForLat:minLat lon:lon];
+                    pa1 = [self pointOnMapForLat:maxLat lon:lon];
+                    minLat = [self freeLatitudeLimitForLongitude:lon+180.0 searchMinLatitude:YES];
+                    maxLat = [self freeLatitudeLimitForLongitude:lon+180.0 searchMinLatitude:NO];
+                    pb0 = [self pointOnMapForLat:minLat lon:lon+180.0];
+                    pb1 = [self pointOnMapForLat:maxLat lon:lon+180.0];
+                    break;
+                }
+                default:
+                    NSLog(@"mapChartWithDate: unknown projection '%c' !", projectionType);
+                    return nil;
+            }
+            for (j=0; j<2; j++)	// MC, IC
+            {
+                p0 = (j==0) ? pa0 : pb0;
+                p1 = (j==0) ? pa1 : pb1;
+                if ( p0.x == MINCOORD || p1.x == MINCOORD )
+                    continue;	// skip lines outside of map
+                color = (j==0) ? [NSColor blueColor]
+                               : [NSColor colorWithCalibratedRed:0.0 green:0.5 blue:1.0 alpha: 1.0];
+
+                line = [VLine lineWithPoints:p0 :p1];
+                [line setColor:color];
+                [line setWidth:lineWidth];
+                [group addObject:line];
+                p = pointOnLineDistanceFromEnd(20.0, p0, p1); // line end (p*1) minus some space
+                [groupMCText addObject:arc = [VArc arcWithCenter:p radius:8.0 filled:YES]];
+                [arc setFillColor:[NSColor yellowColor]];
+                [groupMCText addObject:[VText text:planetSymbol((!j) ? @"MC" : @"IC") center:p
+                                             color:color font:symbolFont14]];
+            }
+        }
+        else
+        {
+            p0 = p0_mir = NSMakePoint(-LARGE_COORD, -LARGE_COORD);
+            if (projectionType == 'M')
+                lonStart = [self latLonForPointOnMap:viewBounds.origin].x;
+            for ( lon =  (!isMCIC)  ? lonStart         : lonMCIC;
+                  lon <= ((!isMCIC) ? (lonStart+360.0) : lonMCIC);
+                  lon += jump)	// segment every 5 deg longitude
+            {   float	latMin = -90.0, latMax = 90.0;
+
+                if (i == 0)
+                {   double	st = vhfSideralTime(jd, lon);
+                    lat = vhfLatForAC(eDeg, st);
+                    tDeg = ( vhfIsAC(lat, eDeg, st) ) ? 0.0 : 180.0;
+                }
+                else
+                    // FIXME: the topocentricDegree function does not work properly
+                    for (n=0; n<20; n++)	// binary search through latitudes
+                    {   double	tDeg1;
+
+                        lat = (latMin+latMax) / 2.0;
+                        [topo setLatitude:lat longitude:lon];
+                        tDeg = [topo topocentricDegree:eDeg];
+                        // if tDeg is close to 180, we have the opposite house
+//printf("binary: lat = %.0f  lon = %.0f  tDeg = %.2f eDeg = %.2f house = %.0f  latMin = %.2f latMax = %.2f\n", lat, lon, tDeg, eDeg, houseDeg, latMin, latMax);
+                        if (Diff(tDeg, houseDeg) < 0.01)
+                            break;	// ready
+                        // larger lat (tDeg1) closer to houseDeg as tDeg -> set latMin
+                        [topo setLatitude:lat+0.1 longitude:lon];
+                        tDeg1 = [topo topocentricDegree:eDeg];
+                        if ( angle180(tDeg, houseDeg) > angle180(tDeg1, houseDeg) )
+                        //if ( vhfIsDegreeInsideRange(tDeg, houseDeg-180.0, houseDeg, NO) )
+                            latMin = lat;		// < target house
+                        else
+                            latMax = lat;		// > target house
+                    }
+                //printf("Equal: lat: %f == %f, tDeg: %f == %f\n", lat, lat1, tDeg, tDeg1);
+                //if (i == 0)
+                //    lat = lat1, tDeg = tDeg1;
+
+                p1     = [self pointOnMapForLat:lat lon:lon];
+                p1_mir = [self pointOnMapForLat:-lat lon:mod360(lon+180.0)];
+
+                if (projectionType != 'S')	// text position close to equator
+                {
+                    if (lon < lonStart+180.0 && Abs(lat) < Abs(textLatLon[0].y))
+                        textLatLon[0] = NSMakePoint(lon, lat);
+                    else if (lon >= lonStart+180.0 && Abs(lat) < Abs(textLatLon[1].y))
+                        textLatLon[1] = NSMakePoint(lon, lat);
+                }
+
+                if ( p0.x > -LARGE_COORD && p1.x > MINCOORD &&
+                     (NSPointInRect(p0, viewBounds) || NSPointInRect(p1, viewBounds)) )	// line inside bounds
+                {
+//printf("lat = %.0f  lon = %.0f  tDeg = %f\n", lat, lon, tDeg);
+                    /* if tDeg changes from 0 to 180, the house tip changes and we stop
+                     */
+                    if (!polyLine[0]
+                        //|| lat > 70.0 || lat < -70.0
+                        || DiffPoint([polyLine[0] pointWithNum:[polyLine[0] numPoints]-1], p0) > TOLERANCE)
+                    {
+                        if ([polyLine[0] numPoints] >= 2)
+                            [group addObject:polyLine[0]];
+                        //if (lat > 70.0 || lat < -70.0)
+                        //    p1 = NSMakePoint(-LARGE_COORD, -LARGE_COORD);	// reset
+                        //if ( lat < 70 && lat > -70 )
+                        {
+                            polyLine[0] = [VPolyLine polyLine];
+                            [polyLine[0] setColor:color];
+                            [polyLine[0] setWidth:lineWidth];
+                            [polyLine[0] addPoint:p0];
+                        }
+                    }
+                    /* lat inside range + tDeg on correct side */
+                    if ( /*lat < 70 && lat > -70 &&*/ Diff(tDeg, houseDeg) < 90.0 )
+                        [polyLine[0] addPoint:p1];
+                }
+                if ( p0_mir.x > -LARGE_COORD && p1_mir.x > MINCOORD &&
+                     (NSPointInRect(p0_mir, viewBounds) || NSPointInRect(p1_mir, viewBounds)) )	// line inside bounds
+                {
+                    if (!polyLine[1]
+                        || Diff(p0_mir.x, p1_mir.x) > 50.0	// flip
+                        || DiffPoint([polyLine[1] pointWithNum:[polyLine[1] numPoints]-1], p0_mir) > TOLERANCE)
+                    {
+                        if ([polyLine[1] numPoints] >= 2)
+                            [group addObject:polyLine[1]];
+                        polyLine[1] = [VPolyLine polyLine];
+                        [polyLine[1] setColor:[houseColors objectAtIndex:i+houseCnt/2]];
+                        [polyLine[1] setWidth:lineWidth];
+                        if (Diff(p0_mir.x, p1_mir.x) < 50.0)	// no flip
+                            [polyLine[1] addPoint:p0_mir];
+                    }
+                    /* lat inside range + tDeg on correct side */
+                    if ( Diff(tDeg, houseDeg) < 90.0 )
+                        [polyLine[1] addPoint:p1_mir];
+                }
+                //if (lat < 70.0 && lat > -70.0)
+                {   p0 = p1;
+                    p0_mir = p1_mir;
+                }
+            }
+            if ([polyLine[0] numPoints] >= 2)
+                [group addObject:polyLine[0]];
+            if ([polyLine[1] numPoints] >= 2)
+                [group addObject:polyLine[1]];
+
+            /* text */
+            {   NSPoint	ps[2] = {{MINCOORD, MINCOORD}, {MINCOORD, MINCOORD}}, *pts;
+
+                /* for stereo projection intersect PolyLine(s) and bounds and
+                 * place text a little away from the intersection points
+                 */
+                if (projectionType == 'S' || projectionType == 'F')	// stereo or free
+                {   int		n, cnt;
+                    NSRect	rect = (bounds.size.width == 0.0) ? NSInsetRect(viewBounds, 2*16, 2*16)
+                                                                  : bounds;
+
+                    for (n=groupCount; n<(int)[[group list] count]; n++)
+                    {   VPolyLine	*g = [[group list] objectAtIndex:n];
+
+                        if ( (cnt = [g intersections:&pts withRect:rect]) )
+                        {
+                            if (cnt >= 1)
+                                ps[0] = pts[0];
+                            if (cnt >= 2)
+                                ps[1] = pts[1];
+                            free(pts);
+                        }
+                        /*else if (projectionType == 'F')	// polyline is too short -> place text at ends
+                        {
+                            ps[0] = [];
+                        }*/
+                    }
+                }
+                else			// merkator, ...
+                {
+                    if (textLatLon[0].x != 180.0)
+                        ps[0] = [self pointOnMapForLat:0.0 lon:textLatLon[0].x];
+                    if (textLatLon[1].x != 180.0)
+                        ps[1] = [self pointOnMapForLat:0.0 lon:textLatLon[1].x];
+                }
+                for (j=0; j<2; j++)
+                {   int	houseNum = (j==0) ? (i+1) : (i+houseCnt/2+1);
+
+                    if (ps[j].x == MINCOORD)
+                        continue;
+                    [groupACText addObject:arc = [VArc arcWithCenter:ps[j] radius:8.0 filled:YES]];
+                    [arc setFillColor:[NSColor yellowColor]];
+                    [groupACText addObject:[VText text:[NSString stringWithFormat:@"%d ", houseNum] center:ps[j]
+                                                 color:[NSColor blackColor] font:systemFont14]];
+                }
+            }
+        }
+    }
+
+    [group addObject:groupMCText];
+    [group addObject:groupACText];
+
+    return group;
+}
+
+/* map time tracks
+ * 1. full circle on map for each earth cycle  (interval << 24h)
+ *    map declination/longitude at maridian
+ * 2. full circle on map for each object cycle (interval ~ object cycle)
+ *    map declination/longitude of object meridian after full earth revolution
+ *
+ * parammeter:
+ *  dateFr	begin of time range
+ *  dateTo	end   of time range
+ *  objects	an array of the object names we have to calculate
+ *  title	
+ *  viewBounds	
+ * 
+ * created:  2006-02-19
+ * modified: 2006-02-20
+ */
+#define STARDAY_SEC	86164.1	// 23:56:04.1
+- (VGroup*)mapTimeTrackFromDate:(NSCalendarDate*)dateFr toDate:(NSCalendarDate*)dateTo
+                     forObjects:(NSArray*)objects earthRevolution:(BOOL)earthRev
+                          title:(NSString*)title inBounds:(NSRect)viewBounds
+{   VGroup		*group = [VGroup group];
+    Ephemeris		*ephemeris = [[AstroPrincipal instance] ephemeris];
+    //EphemerisSearch	*ephemerisSearch = nil;
+    //NSArray		*searchArray = nil;
+    VPolyLine		*polyLine[[objects count]];
+    int			objectIx[[objects count]], i, cnt = 0, objIx;
+    NSTimeInterval	timeRange = [dateTo timeIntervalSinceDate:dateFr];
+    EphemerisFlags	ephFlags = EPHFLAG_LON + EPHFLAG_DEC + EPHFLAG_LOC;	// what to calculate
+    EphemerisObjects	ephObjIx[[objects count]];	// objects to calculate
+    int			ephObjCnt = 0;			// number of objects in array
+    int			interval = (int)STARDAY_SEC;	// interval in [min]
+    NSCalendarDate	*date = dateFr;
+    //BOOL		topoFlag = Prefs_Topocentric;
+    float		lat = 0.0, lon = 0.0;
+    int			elev = 0;
+    NSPoint		p, pPrev[[objects count]];
+    float		lineWidth = 1.0;
+    NSFont		*font = [NSFont fontWithName:Prefs_AstroFont size:20.0];
+
+    if (!ephemeris)
+        return nil;
+
+    /* translation tables (object array -> ephemeris index) */
+    for (i=0; i<[objects count]; i++)	// translation of indexes from objects array to ephemeris
+    {   objectIx[i] = [ephemeris indexForObjectName:[objects objectAtIndex:i]];
+        polyLine[i] = nil;
+        pPrev[i] = NSZeroPoint;
+    }
+
+    /* prepare ephemeris indexes to calculate */
+    for (i=0; i<[objects count]; i++)
+        ephObjIx[ephObjCnt++] = objectIx[i];
+
+    /* search object position for start (Ecliptic, Aspect, ...) */
+    /*if (!earthRev)
+    {
+        if (!ephemerisSearch)
+            ephemerisSearch = [[EphemerisSearch newWithEphemeris:ephemeris] autorelease];
+        //[ephemerisSearch setProgressTarget:progressTarget action:progressAction];
+        [ephemerisSearch setTopocentricSearch:topoFlag lat:lat lon:lon elev:elev];
+        searchArray = [[ephemerisSearch searchFromDate:dateFr toDate:dateTo
+                                               object1:objectIx[0] object2:EPH_MC
+                                                aspect:S_ASPECT_ANGLE aspectValue:0.0] retain];
+    }*/
+
+    /* set features */
+    interval = (!earthRev) ? (int)STARDAY_SEC : (60*10);	// [sec]
+    if (interval < 5*60)	// at least 5 min steps
+        interval = 5*60;
+
+    /* step through time range
+     * get longitude and declination where MC is on object
+     * get point on map for declination (= latitude) and longitude
+     */
+    cnt = (int)timeRange / interval;
+    for (i=0; i<cnt; i++)	// loop over time range
+    {   double		tjd = [ephemeris julianDayForDate:date bc:NO];
+        EphObject	*ephObjects;
+
+        lat = 0.0; lon = 0.0; elev = 0;
+        ephObjects = [ephemeris objectsAtJD:tjd flags:ephFlags
+                                    objects:ephObjIx cnt:ephObjCnt
+                                  longitude:lon latitude:lat elevation:elev topocentric:NO];
+
+        //[progressTarget performSelector:progressAction
+        //                     withObject:[NSNumber numberWithFloat:(float)loop++/(float)loopCnt]];
+
+        for ( objIx = 0; objIx < [objects count]; objIx++ )	// objects
+        {
+            if (objectIx[objIx] == EPH_MC)	// skip MC
+                continue;
+            if (!polyLine[objIx])
+                polyLine[objIx] = [VPolyLine polyLine];
+
+            lat = ephObjects[objectIx[objIx]].dec;
+            if ( projectionType == 'S' )	// move curve to visible latitude, FIXME: North/South !
+            {   lat += 60.0;
+                //lat = Abs(lat);	// this is what the weather does
+            }
+            lon = vhfLonForMC(ephObjects[objectIx[objIx]].lon, date);	// longitude of object
+            // FIXME: topocentric positions
+            p = [self pointOnMapForLat:lat lon:lon];
+            if ( p.x > MINCOORD &&
+                 (NSPointInRect(p,            viewBounds) ||		// p or previos p inside bounds
+                  NSPointInRect(pPrev[objIx], viewBounds)) &&
+                 Diff(p.x, pPrev[objIx].x) < viewBounds.size.width /2.0 &&	// no flip X - FIXME: close gap
+                 Diff(p.y, pPrev[objIx].y) < viewBounds.size.height/2.0 )	// no flip Y - FIXME: close gap
+                [polyLine[objIx] addPoint:p];
+            else if ([polyLine[objIx] numPoints] >= 2)			// outside -> end the polyline
+            {   NSColor	*color = [colors objectForKey:[objects objectAtIndex:objIx]];
+
+                [polyLine[objIx] setColor:([objects count] == 1) ? [NSColor blueColor] : color];
+                [polyLine[objIx] setWidth:lineWidth];
+                [group addObject:polyLine[objIx]];
+                polyLine[objIx] = nil;
+            }
+            else if ([polyLine[objIx] numPoints])
+                polyLine[objIx] = nil;
+            pPrev[objIx] = p;
+        }
+
+        date = [date dateByAddingYears:0 months:0 days:0 hours:0 minutes:0 seconds:interval];
+    }
+
+    for ( objIx = 0; objIx < [objects count]; objIx++ )	// objects
+    {
+        if ([polyLine[objIx] numPoints])
+        {   NSString	*planet = [objects objectAtIndex:objIx];
+            NSColor	*color = ([objects count] == 1) ? [NSColor blueColor]
+                                                        : [colors objectForKey:planet];
+
+            [polyLine[objIx] setColor:color];
+            [polyLine[objIx] setWidth:lineWidth];
+            [group addObject:polyLine[objIx]];
+
+            /* attach symbol to curve */
+            p = [polyLine[objIx] pointWithNum:MAXINT];
+            [group addObject:[VText text:planetSymbol(planet) center:p
+                                   color:color angle:0.0 center:p font:font]];
+        }
+    }
+
+    return group;
 }
 
 
@@ -434,6 +878,7 @@ NSLog(@"1: lat=%f l1=%f -> p={%.2f %.2f}", lat, lon, p.x, p.y);
 
     return NSZeroPoint;
 }
+
 /*
  * Mercator projection
  *
@@ -535,7 +980,7 @@ NSLog(@"1: lat=%f l1=%f -> p={%.2f %.2f}", lat, lon, p.x, p.y);
         lon = lon-360.0;
 
     /* latitude = stereographic (azimut) projection */
-    d = 6356752.0 * 2.0 * 72000.0 / 25.0;	// earth diameter in point
+    d = EarthRadiusA * 2.0 * 72000.0 / 25.4;	// earth diameter in point
     x = mapP.x - origin.x;
     y = mapP.y - origin.y;
 
@@ -557,8 +1002,8 @@ NSLog(@"1: lat=%f l1=%f -> p={%.2f %.2f}", lat, lon, p.x, p.y);
     deg = lon + deg0;
 
     /* radius of latitude = stereographic (azimut) projection */
-    d = 6356752.0 * 2.0 * 72000.0 / 25.0;	// earth diameter in point
-    b = (90.0 - lat) / 2.0;
+    d = EarthRadiusA * 2.0 * 72000.0 / 25.4;	// earth diameter in point
+    b = (90.0 - (double)lat) / 2.0;
     rad = Tan(b) * d / scale;
 
     p = vhfPointRotatedAroundCenter(NSMakePoint(origin.x+rad, origin.y), deg, origin);
@@ -769,15 +1214,19 @@ NSPoint vhfProjectedPointInPolygon(NSPoint p, NSPoint *pts, NSPoint *ptsOut)
 - (NSMutableDictionary*)createMap
 {   NSMutableDictionary	*dict;
 
-    if ( projectionType )	// allready calculated
+    if ( scale && projectionType )	// allready calculated
         return nil;
     if (!ciaMap)
         ciaMap = [[CIAMap map] retain];
 NSLog(@"%f %f %f %f", center.x, center.y, size.width, size.height);
-    dict = [ciaMap generateMapListWithCenter:center size:size grid:10.0 :NO :NO mapset:mapset];
+
+    if (!projectionType)
+        projectionType = 'M';
+    dict = [ciaMap generateMapListWithCenter:center size:size rotation:deg0
+                                        grid:10.0 projection:projectionType
+                                            :NO :NO mapset:mapset];
 
     /* set parameter */
-    projectionType = 'M';
     origin = [ciaMap origin];
     scale = [ciaMap scale];
 NSLog(@"createMap: scale = %f", scale);
@@ -791,9 +1240,10 @@ NSLog(@"createMap: scale = %f", scale);
 
 
 /* load map image and add background to map
+ * modified: 2006-04-03
  */
 - (void)displayMapAtDate:(NSCalendarDate*)newMapDate inView:(DocView*)view
-{   NSString	*fileName;
+{   NSString	*fileName = nil;
     NSString	*path = [mapDict objectForKey:@"path"];
     int		layerBG  = ([mapDict objectForKey:@"bgImageLayer"]) ? [mapDict intForKey:@"bgImageLayer"] : 0;
     int		layerMap = ([mapDict objectForKey:@"mapLayer"])     ? [mapDict intForKey:@"mapLayer"]     : 1;
@@ -801,13 +1251,14 @@ NSLog(@"createMap: scale = %f", scale);
     VImage	*image;
     LayerObject	*layer = [[view layerList] objectAtIndex:layerMap];
     NSArray	*mapTimes = [mapDict objectForKey:@"mapTimes"];
+    NSArray	*mapPress = [mapDict objectForKey:@"pressure"];
 
     if (![mapDict objectForKey:@"fileFormat"])
         return;
 
     /* find map closest to available maps
      */
-    if ([mapTimes count])
+    if ([mapTimes count])	// surface maps
     {   int	hourTarget = [newMapDate hourOfDay]*100 + [newMapDate minuteOfHour];	// HHMM
         int	minDiff = 9999, bestI = 0, i, hour;
 
@@ -826,22 +1277,65 @@ NSLog(@"createMap: scale = %f", scale);
                                              hour:hour/100
                                            minute:hour-(hour/100*100)
                                            second:0 timeZone:[NSTimeZone timeZoneWithName:@"UTC"]];
+        fileName = [newMapDate descriptionWithCalendarFormat:[mapDict objectForKey:@"fileFormat"]];
     }
+    if ([mapPress count])	// upper air maps
+    {   NSArray	*layerList = [view layerList];
+        int	i, l;
+
+        newMapDate = [NSCalendarDate dateWithYear:[newMapDate yearOfCommonEra]
+                                            month:[newMapDate monthOfYear]
+                                              day:[newMapDate dayOfMonth]
+                                             hour:12
+                                           minute:0
+                                           second:0 timeZone:[NSTimeZone timeZoneWithName:@"UTC"]];
+        /* find enabled layer/page with substring from pressure array */
+        layer = nil;
+        for (i=0; i<[mapPress count]; i++)
+        {   NSString	*idStr = [mapPress objectAtIndex:i];
+
+            for (l=0; l<[layerList count]; l++)
+            {
+                if ( [[[layerList objectAtIndex:l] string] rangeOfString:idStr].length &&
+                     [[layerList objectAtIndex:l] state] )
+                {   NSString	*formatStr;
+
+                    layer = [layerList objectAtIndex:l];
+                    /* add prefix (pressure) to file name */
+                    formatStr = [[mapDict objectForKey:@"fileFormat"] stringByReplacing:@"%@" by:idStr];
+                    fileName = [newMapDate descriptionWithCalendarFormat:formatStr];
+                    // FIXME: if prefix/layer has changed we have to reset mapDate to load the image
+                    //if ( prefix changed )
+                    //    mapDate = nil;
+                    break;
+                }
+            }
+            if ( l<[layerList count] )
+                break;
+        }
+        if ( !layer )
+            return;
+    }
+    if (!fileName)
+        fileName = [newMapDate descriptionWithCalendarFormat:[mapDict objectForKey:@"fileFormat"]];
 
-    fileName = [newMapDate descriptionWithCalendarFormat:[mapDict objectForKey:@"fileFormat"]];
     if ( [path rangeOfString:@"%@"].length > 0 )	// substitute year
         path = [NSString stringWithFormat:path, [newMapDate descriptionWithCalendarFormat:@"%Y"]];
-    if (![path hasPrefix:@"/"])	// relative path -> add user library
+    if (![path hasPrefix:@"/"])				// relative path -> add user library
         path = [userLibrary() stringByAppendingPathComponent:path];
     path = [path stringByAppendingPathComponent:fileName];
 
     if ( ![[NSFileManager defaultManager] fileExistsAtPath:path] )
-    {   NSLog(@"Map -displayMapAtDate: Can't load image from file %@", path);
+    {   static BOOL	displayMessage = YES;
+
+        if (displayMessage)
+            NSLog(@"Map -displayMapAtDate: Can't load image from file %@", path);
+        displayMessage = NO;
         return;
     }
 
     if (mapDate && [mapDate isEqual: newMapDate])
-        return;
+        return;	// allready loaded
     [mapDate release];
     mapDate = [newMapDate retain];
 
@@ -903,9 +1397,10 @@ NSLog(@"createMap: scale = %f", scale);
     [image scale:0.5 :0.5 withCenter:[image origin]];
     if ([view scaleFactor] == 2.0)
         [[image image] recache];	// needed for 1:1 scale to make background visible
-    [layer removeAllObjects];
-    [layer addObject:image];
-
+    if (image)
+    {   [layer removeAllObjects];
+        [layer addObject:image];
+    }
     [view drawAndDisplay];
 }
 
@@ -944,7 +1439,7 @@ NSLog(@"createMap: scale = %f", scale);
                 //[mapDict setObject:widthLat forKey:@"widthLat"];
             }
             break;
-        case 'S':	// Stereographic
+        case 'S':	// Stereographic / Azimut
             [mapDict setObject:[NSString stringWithFormat:@"%f", origin.x] forKey:@"x"];
             [mapDict setObject:[NSString stringWithFormat:@"%f", origin.y] forKey:@"y"];
             [mapDict setObject:[NSString stringWithFormat:@"%f", scale] forKey:@"scale"];
@@ -969,12 +1464,14 @@ NSLog(@"createMap: scale = %f", scale);
 
 - (void)dealloc
 {
+    [colors release];
+
     [mapDict release];
     [name release];
     [mapset release];
     [freeGrid release];
 
-    [date release];
+    [dateAxes release];
     [mapDate release];
 
     [super dealloc];
