@@ -1,16 +1,16 @@
 /* TopocentricCycle.m
  * Topocentric houses and related functions
  *
- * This file uses some functions from the
+ * This class uses some starters and knowhow from
  * Astrolog free Astrology software by Walter D. Pullen
  * (http://www.magitech.com/~cruiser1/astrolog.htm)
  *
- * The extensions are
- * Copyright (C) 1999-2005 by vhf interservice GmbH
+ * The cycle extensions are
+ * Copyright (C) 1999-2006 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  2003-08-11
- * modified: 2005-04-25 (clean up)
+ * modified: 2005-12-12
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -46,6 +46,12 @@
     return [cycle autorelease];
 }
 
+- (id)init
+{
+    isDirty = YES;
+    return [super init];
+}
+
 - (id)initWithDate:(NSCalendarDate*)utc latitude:(float)lat longitude:(float)lon
 {
     [super init];
@@ -58,7 +64,7 @@
  */
 static double RSgn(double r)
 {
-  return r == 0.0 ? 0.0 : ((r) < 0.0 ? -1.0 : 1.0);
+    return r == 0.0 ? 0.0 : ((r) < 0.0 ? -1.0 : 1.0);
 }
 
 /* Modulus function to bring the given
@@ -186,13 +192,19 @@ static double MinDifference(double deg1, double deg2)
 
 /* return topocentric houses with variable number of houses
  * cnt%4 == 0
- * created: 1999-11-30
+ * created:  1999-11-30
+ * modified: 2005-12-12 (dirty flag added)
  */
 - (double*)houses:(int)cnt
 {   double		tl, savedLat = topo.lat, m, m1 = 0.0;
+    static int		lastCnt = -1;
     static double	houses[360];
     int			i;
 
+    if ( !isDirty && cnt == lastCnt )
+        return houses;
+    lastCnt = cnt;
+
     houses[cnt/4] = ModRad(DegToRad(topo.MC + 180.0));
     tl = tan(topo.lat);
 
@@ -224,6 +236,8 @@ static double MinDifference(double deg1, double deg2)
         houses[i+cnt/2] = Mod(houses[i] + 180.0);
     }
 
+    isDirty = NO;
+
     return houses;
 }
 
@@ -272,6 +286,7 @@ static double MinDifference(double deg1, double deg2)
     /* Flip the Ascendant if it falls in the wrong half of the zodiac. */
     if (MinDifference(topo.MC, topo.AC) < 0.0)
         topo.AC = Mod(topo.AC + 180.0);
+    isDirty = YES;
 }
 
 - (void)setLatitude:(float)lat longitude:(float)lon
@@ -293,6 +308,7 @@ static double MinDifference(double deg1, double deg2)
     /* Flip the Ascendant if it falls in the wrong half of the zodiac. */
     if (MinDifference(topo.MC, topo.AC) < 0.0)
         topo.AC = Mod(topo.AC + 180.0);
+    isDirty = YES;
 }
 
 /* return topocentric houses with variable number of houses
@@ -338,8 +354,6 @@ static double MinDifference(double deg1, double deg2)
 /* return local position for ecliptic position.
  * AC = 0, 2 = 30, 3 = 60, 4 = 90, ... MC = 270, ...
  * created: 2003-08-11
- * FIXME: the algorithm is just a quick and dirty approximation of what we really want.
- *        It should be constructed directly using trigonometric functions.
  */
 - (double)topocentricDegree:(double)eclipticDeg
 {   int		i;
@@ -352,7 +366,7 @@ static double MinDifference(double deg1, double deg2)
         return 90.0;
 
     eDegMin = [self eclipticDegree:tDegMin];
-    for (i=0; i<20; i++)
+    for (i=0; i<20; i++)	// binary search
     {
         tDeg = (tDegMin + tDegMax) / 2.0;
         eDeg = [self eclipticDegree:tDeg];
