@@ -1,11 +1,11 @@
 /* astroCommon.h
  * Common astronomical functions
  *
- * Copyright (C) 1998-2005 by vhf interservice GmbH
+ * Copyright (C) 1998-2006 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1998-11-06
- * modified: 2005-03-15
+ * modified: 2006-02-02
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -29,8 +29,6 @@
 #ifndef VHF_H_ASTROCOMMON
 #define VHF_H_ASTROCOMMON
 
-#define CALENDARFORMAT	@"%d.%m.%Y-%H:%M-%z"
-
 float sunNode(int year);
 
 NSMutableData* reducedTable(NSData *tableData, int tableSize, double factor);
@@ -47,33 +45,61 @@ double harmonicValue(double deg);
 //float angleFromFirstToSecond( float deg1, float deg2 );
 //float angleBetween( float deg1, float deg2 );
 
+/* degree/string conversion */
 NSString *stringFromDeg(float deg);
 float degFromString(NSString *degString);
 
 int sortAsNumbers(id str1, id str2, void *context);
 
 float frac(float x);
+
+/* modified julian day (reference = 1851) */
+double vhfMJD(NSCalendarDate *date);
+double vhfSideralTimeMJD(double mjd, float lon);
+
+/* Julian Day (reference = -4712, compatible to swiss ephemeris) */
+double vhfJulianDay(NSCalendarDate *date);
+double vhfSideralTime(double jd, float lon);	// sideral time in hours
+
+/* calculate astronomical values with sideral time [hours] (or better seconds and integer ???) */
+double vhfRA(double sideralTime);
+double vhfMC(double sideralTime);
+double vhfAC(double sideralTime, float lat);
+BOOL vhfIsAC(float lat, float acDeg, double st);
+double vhfSideralTimeForMC(float mcDeg, NSCalendarDate *utc);
+float vhfLonForSideralTime(double st, NSCalendarDate *gmt);
+float vhfLonForMC(float mcDeg, NSCalendarDate *utc);
+float vhfLatForAC(float acDeg, double st);
+
+/* calculate astronomical values using NSCalendarDate (DEPRECATED !) */
 NSCalendarDate *sideralTime(NSCalendarDate *gmt, float longitude);
-
-float ra(NSCalendarDate *sideralTime);
 float mc(NSCalendarDate *sideralTime);
-NSCalendarDate *sideralTimeWithMC(float mcDeg, NSCalendarDate *utc);
 float ac(NSCalendarDate *sideralTime, float latitude);
+//float ra(NSCalendarDate *sideralTime);
+//NSCalendarDate *sideralTimeWithMC(float mcDeg, NSCalendarDate *utc);
+//BOOL isAC(float lat, float acDeg, NSCalendarDate *sideralTime);
+//float longitudeForMC(float mcDeg, NSCalendarDate *sideralTime);
+//float latitudeForAC (float acDeg, NSCalendarDate *sideralTime);
 
-BOOL isAC(float lat, float acDeg, NSCalendarDate *sideralTime);
-float longitudeForMC(float mcDeg, NSCalendarDate *sideralTime);
-float latitudeForAC(float acDeg, NSCalendarDate *sideralTime);
-
+/* object symbols  */
 NSString *planetSymbol(NSString *object);
 NSString *nodeSymbol(NSString *planet);
 NSString *zodiacSign(int sign);
 NSString *aspectSymbol(float deg, float orb);
 
-double standardAngle( double angle );
-double angle(double deg1, double deg2);
+/* modulus */
+double mod360(double deg);			// modulus 360 degree
+
+/* aspects */
+double standardAngle( double angle );		// modulus 360 degree
+double angle(double deg1, double deg2);		// angle = [0, 360]
+double angle180( double deg1, double deg2 );	// angle = [0, 180]
 double mirror(double deg1, double deg2);
 double midpoint(double deg1, double deg2);
 double harmony(double obj, double obj1, double obj2);
 
+/* checks */
+BOOL vhfIsDegreeInsideRange(double deg, double degFr, double degTo, BOOL retro);
+
 
 #endif // VHF_H_ASTROCOMMON
