@@ -1,11 +1,11 @@
 /* astroCommon.m
  * Common astronomical functions
  *
- * Copyright (C) 1998-2005 by vhf interservice GmbH
+ * Copyright (C) 1998-2006 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1998-11-06
- * modified: 2005-03-15 (midpoint())
+ * modified: 2006-04-06 (Pholus ']' added, Pallas '[' corrected, new astro functions without NSCalendarDate)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -321,15 +321,15 @@ double *harmonicPattern1(int tableSize)
 float degFromString(NSString *degString)
 {   float		deg = 0.0;
     NSScanner		*scanner = [NSScanner scannerWithString:degString];
-    NSCharacterSet	*digitSet = [NSCharacterSet decimalDigitCharacterSet];
+    NSCharacterSet	*digitSet  = [NSCharacterSet decimalDigitCharacterSet];
     NSCharacterSet	*degreeSet = [NSCharacterSet characterSetWithCharactersInString:@" '\""];
     NSString		*string;
     NSRange		range;
 
     range = [degString rangeOfCharacterFromSet:degreeSet];
-    if ( !range.length )
+    if ( !range.length )	// 4.5
         deg = [degString floatValue];
-    else
+    else			// 4 30'00"
     {
         if ( [scanner scanCharactersFromSet:digitSet intoString:&string] )
             deg += [string floatValue];
@@ -343,7 +343,7 @@ float degFromString(NSString *degString)
     return deg;
 }
 
-/* created: 21.08.97
+/* created: 1997-08-21
  */
 int sortAsNumbers(id str1, id str2, void *context)
 {
@@ -354,9 +354,9 @@ int sortAsNumbers(id str1, id str2, void *context)
     return NSOrderedSame;
 }
 
-/* MJD: modified julian date
+/* MJD: modified julian day (starting in 1858)
  */
-double mjd( NSCalendarDate *date )
+double vhfMJD( NSCalendarDate *date )
 {   int		year, month, day;
     double	hour, mjd;
     double	a, b;
@@ -368,8 +368,7 @@ double mjd( NSCalendarDate *date )
 
     a = 10000.0*year + 100.0*month + day;
     if ( month <= 2 )
-    {
-        month += 12;
+    {   month += 12;
         year--;
     }
     if ( a <= 15821004.1 )
@@ -411,6 +410,46 @@ NSCalendarDate *dateFromMJD(double mjd)
     return date;
 }
 
+double vhfSideralTimeMJD(double mjd, float lon)
+{   double	mj = mjd, mjd0, t, ut, gmst, lmst;
+
+    mjd0 = floor( mj );
+    ut = (mj-mjd0)*24;
+    t = (mjd0-51544.5)/36525.0;
+    gmst = 6.697374558+1.0027379093*ut+(8640184.812866+(0.093104-6.2e-6*t)*t)*t/3600.0;
+    lmst = 24.0*frac((gmst+lon/15.0)/24.0);
+    return lmst;	// hours
+}
+
+
+/* Julian day (starting -4712)
+ */
+double vhfJulianDay(NSCalendarDate *date)
+{   int		year, month, day;
+    double	hour, y0, y1, jd;
+
+    year  = [date yearOfCommonEra];
+    month = [date monthOfYear];
+    day   = [date dayOfMonth];
+    hour  = (double)[date hourOfDay]+[date minuteOfHour]/60.0+[date secondOfMinute]/3600.0;
+
+    if ( month <= 2 )
+    {   month += 12;
+        year--;
+    }
+    y0 = year;
+    jd = floor((y0+4712.0)*365.25) + floor(30.6*(month+1.0)+0.000001) + day + hour/24.0 - 63.5;
+
+    y1 = Abs(year)/100 - Abs(year)/400;
+    if (year < 0)
+        y1 = -y1;
+    jd = jd - y1 + 2.0;
+    if ((year < 0) && (y0/100.0 == floor(y0/100.0)) && (y0/400.0 != floor(y0/400.0)))
+        jd -= 1.0;
+
+    return jd;
+}
+
 /* gmt to local time
  * start at 22.09. 00:00 UTC
  * each 24h (86400s) +4 minutes (star day = 23:56:04.1) -> +235.9 sec
@@ -422,28 +461,183 @@ float frac(float x)
     if (x<0) x += 1.0;
     return x;
 }
+
+
+/* sideral time in hours with comma (reference date = -4712)
+ */
+double vhfSideralTime(double jd, float lon)
+{   double	mj = jd, mjd0, t, ut, gmst, lmst;
+
+    mjd0 = floor(mj);
+    ut = (mj-mjd0-0.5) * 24.0;
+    t = (mjd0-2451544.5) / 36525.0;
+    gmst = 6.697374558+1.0027379093*ut+(8640184.812866+(0.093104-6.2e-6*t)*t)*t/3600.0;
+    lmst = 24.0*frac((gmst+lon/15.0)/24.0);
+    return lmst;	// hours
+}
+
+float vhfLonForSideralTime(double st, NSCalendarDate *gmt)
+{   double	mj, mjd0, t, ut, gmst, lon;
+
+    mj = vhfMJD(gmt);
+    mjd0 = floor(mj);
+    ut = (mj-mjd0)*24;
+    t = (mjd0-51544.5)/36525.0;
+    gmst = 6.697374558+1.0027379093*ut+(8640184.812866+(0.093104-6.2e-6*t)*t)*t/3600.0;
+    lon = (st - gmst) * 15.0;
+
+    while (lon <= -180.0)
+        lon += 360.0;
+    while (lon > 180.0)
+        lon -= 360.0;
+
+    return lon;
+}
+
+double vhfRA(double st)
+{
+    return st * 15.0;
+}
+
+double vhfMC(double st)
+{   double	l, mc, ra;
+
+    ra = st * 15.0;	// secs / (4.0*60.0);
+    l = COS_E * Cot(ra);
+    mc = Acot(l);
+    if ( mc < 0.0 )
+        mc += 360.0;
+
+    /* from 12:00 - 00:00 sideralTime -> mc = 180 - 360 degree
+     * from 00:00 - 12:00 sideralTime -> mc = 0 - 180 degree
+     * -06:00 -> mc
+     * 06:00- -> mc -= 180.0
+     */
+    if ( st >= 0.0 && st < 12.0 )
+    {
+        if ( mc > 180.0 )
+            mc -= 180.0;
+    }
+    else if ( mc < 180.0 )
+        mc += 180.0;
+
+    if ( mc < 0.0 )
+        mc += 360.0;
+    else if ( mc >= 360.0 )
+        mc -= 360.0;
+
+    return mc;
+}
+
+double vhfAC(double st, float lat)
+{   double	oa, ac, m, a;
+
+    oa = st * 15.0 + 90.0;	// secs / (4.0*60.0) + 90.0
+    ac  = Acot( COS_E*Cot(oa) - SIN_E*Tan(lat)/Sin(oa) );
+    if ( ac < 0 )
+        ac += 360.0;
+
+    /* AC is ccw from MC */
+    m = vhfMC(st);
+    ac = ((a = angle(ac, m)) < 180.0) ? ac : ac+180.0;
+    if ( ac >= 360.0 )
+        ac -= 360.0;
+
+    /* from 18:00 - 06:00 sideralTime -> ac = 0 - 180 degree
+     * from 06:00 - 18:00 sideralTime -> ac = 180 - 360 degree
+     * these are the points where all ACs go through, but not the turn around points !!!
+     */
+    /*if ( s>=21600 && s<64800 )
+    {
+        if ( ac<180.0 )
+            ac += 180.0;
+    }
+    else if ( ac>180.0 )
+        ac -= 180.0;*/
+
+    return ac;
+}
+
+/* get latitude for AC or DC degree
+ */
+BOOL vhfIsAC(float lat, float acDeg, double st)
+{   float	a = angle(vhfAC(st, lat), acDeg);
+
+    if ( a < 10.0 || a > 350.0)
+        return YES;
+    return NO;
+}
+
+/* MC + UTC -> sideral time
+ * FIXME: we don't need UTC for the sideral time offset (without date) we return
+ */
+double vhfSideralTimeForMC(float mcDeg, NSCalendarDate *utc)
+{   double	l, mc, ra, st;
+
+    mc = mcDeg;
+    /*if (mcDeg > 180.0)
+        mc -= 180.0;*/
+
+    l = Cot(mc);
+    ra = Acot(l / COS_E);
+    if (mcDeg >= 90.0 && mcDeg < 270.0)
+        ra += 180.0;
+    else if (mcDeg >= 270)
+        ra += 360.0;
+    st = ra * (4.0*60.0) / 3600.0;
+
+    return st;
+}
+
+/* the opposite direction for AC/MC
+ * MC + UTC -> longitude
+ */
+float vhfLonForMC(float mcDeg, NSCalendarDate *utc)
+{   double	st;
+    float	lon;
+
+    st  = vhfSideralTimeForMC(mcDeg, utc);
+    lon = vhfLonForSideralTime(st, utc);
+    return lon;
+}
+
+/* this returns either AC or DC. check later with isAC()
+ */
+float vhfLatForAC(float acDeg, double st)
+{   double	l, oa, lat;
+
+    oa = (st * 3600.0) / (4.0*60.0) + 90.0;
+    /* FIXME: if acDeg == 0.0 -> Cot(acDeg) gets infinity */
+    l = Cot(acDeg);
+    lat = Atan( (COS_E*Cot(oa) - l) * Sin(oa) / SIN_E );
+
+    return lat;
+}
+
+
+
+/*
+ * NSCalendarDate functions (slower)
+ */
+
 NSCalendarDate *sideralTime(NSCalendarDate *gmt, float longitude)
 {   double	mj, mjd0, t, ut, gmst, lmst;
 
-    longitude = -longitude;
-
-    mj = mjd(gmt);
-//NSLog(@"gmt:%@ gmt1:%@", gmt, dateFromMJD(mj));
-
+    mj = vhfMJD(gmt);
     mjd0 = floor( mj );
     ut = (mj-mjd0)*24;
     t = (mjd0-51544.5)/36525.0;
     gmst = 6.697374558+1.0027379093*ut+(8640184.812866+(0.093104-6.2e-6*t)*t)*t/3600.0;
-    lmst = 24.0*frac((gmst-longitude/15.0)/24.0);
+    lmst = 24.0*frac((gmst+longitude/15.0)/24.0);
 
     gmt = [NSCalendarDate dateWithString: [gmt descriptionWithCalendarFormat:@"%Y%m%d 00:00:00 %z"]
                           calendarFormat:@"%Y%m%d %H:%M:%S %z"];
     return [gmt dateByAddingYears:0 months:0 days:0 hours:0 minutes:0 seconds:(int)(lmst*3600.0)];
 }
-float longitudeWithSideralTime(NSCalendarDate *st, NSCalendarDate *gmt)
+/*float longitudeWithSideralTime(NSCalendarDate *st, NSCalendarDate *gmt)
 {   double	mj, mjd0, t, ut, gmst, lmst, lon;
 
-    mj = mjd(gmt);
+    mj = vhfMJD(gmt);
     mjd0 = floor( mj );
     ut = (mj-mjd0)*24;
     t = (mjd0-51544.5)/36525.0;
@@ -459,15 +653,15 @@ float longitudeWithSideralTime(NSCalendarDate *st, NSCalendarDate *gmt)
         lon -= 360.0;
 
     return lon;
-}
+}*/
 
-float ra( NSCalendarDate *sideralTime)
+/*float ra( NSCalendarDate *sideralTime)
 {   float	s, ra;
 
     s = [sideralTime timeIntervalSinceDate:[NSCalendarDate dateWithString:[sideralTime descriptionWithCalendarFormat:@"%Y.%m.%d-00:00 %z"] calendarFormat:@"%Y.%m.%d-%H:%M %z"]];
     ra = s / (4.0*60.0);
     return ra;
-}
+}*/
 
 float mc( NSCalendarDate *sideralTime)
 {   float	s, l, mc, ra;
@@ -497,17 +691,15 @@ float mc( NSCalendarDate *sideralTime)
     else if ( mc >= 360.0 )
         mc -= 360.0;
 
-//NSLog(@"mc(): -> mc=%f   s = 0 >= %f < 43200", mc, s);
-//NSLog(@"in -> st=%@ s=%f, ra=%f, l=%f, mc=%f", [sideralTime descriptionWithCalendarFormat:@"%Y%m%d%H%M%S %Z"], s, ra, l, mc);
     return mc;
 }
-NSCalendarDate *sideralTimeWithMC(float mcDeg, NSCalendarDate *utc)
+/*static NSCalendarDate *sideralTimeWithMC(float mcDeg, NSCalendarDate *utc)
 {   float		s, l, mc, ra;
     NSCalendarDate	*st;
 
     mc = mcDeg;
-    /*if (mcDeg > 180.0)
-        mc -= 180.0;*/
+    //if (mcDeg > 180.0)
+    //    mc -= 180.0;
 
     l = Cot(mc);
     ra = Acot(l / COS_E);
@@ -519,10 +711,9 @@ NSCalendarDate *sideralTimeWithMC(float mcDeg, NSCalendarDate *utc)
     st = [[NSCalendarDate dateWithString:[utc descriptionWithCalendarFormat:@"%Y%m%d 00:00:00 %z"]
                           calendarFormat:@"%Y%m%d %H:%M:%S %z"]
           dateByAddingYears:0 months:0 days:0 hours:0 minutes:0 seconds:s];
-//NSLog(@"ou -> st=%@ s=%f, ra=%f, l=%f, mc=%f", [st descriptionWithCalendarFormat:@"%Y%m%d%H%M%S %Z"], s, ra, l, mc);
 
     return st;
-}
+}*/
 
 float ac( NSCalendarDate *sideralTime, float lat)
 {   double	s, oa, ac, m, a;
@@ -551,43 +742,42 @@ float ac( NSCalendarDate *sideralTime, float lat)
     else if ( ac>180.0 )
         ac -= 180.0;*/
 
-//NSLog(@"ac(): -> ac=%f  mc=%f a=%f   s = 21600 >= %f < 64800   lat=%f", ac, m, a, s, lat);
     return ac;
 }
 
 /* get latitude for AC or DC degree
  */
-BOOL isAC(float lat, float acDeg, NSCalendarDate *sideralTime)
+/*BOOL isAC(float lat, float acDeg, NSCalendarDate *sideralTime)
 {   float	a = angle(ac(sideralTime, lat), acDeg);
 
     if ( a < 10.0 || a > 350.0)
         return YES;
     return NO;
-}
+}*/
 
 /* the opposite direction for AC/MC
  */
-float longitudeForMC(float mcDeg, NSCalendarDate *utc)
+/*float longitudeForMC(float mcDeg, NSCalendarDate *utc)
 {   NSCalendarDate	*sideralTime;
     float		lon;
 
     sideralTime = sideralTimeWithMC(mcDeg, utc);
     lon = longitudeWithSideralTime(sideralTime, utc);
     return lon;
-}
+}*/
 /* this returns either AC or DC. check later with isAC()
  */
-float latitudeForAC(float acDeg, NSCalendarDate *sideralTime)
+/*float latitudeForAC(float acDeg, NSCalendarDate *sideralTime)
 {   double	s, l, oa, lat;
 
     s = [sideralTime timeIntervalSinceDate:[NSCalendarDate dateWithString:[sideralTime descriptionWithCalendarFormat:@"%Y.%m.%d-00:00-%z"] calendarFormat:@"%Y.%m.%d-%H:%M-%z"]];
     oa = s / (4.0*60.0) + 90.0;
-    /* FIXME: if acDeg == 0.0 -> Cot(acDeg) gets infinity */
+    // FIXME: if acDeg == 0.0 -> Cot(acDeg) gets infinity
     l = Cot(acDeg);
     lat = Atan( (COS_E*Cot(oa) - l) * Sin(oa) / SIN_E );
 
     return lat;
-}
+}*/
 
 /* convert planet string to planet symbol
  */
@@ -626,9 +816,11 @@ NSString *planetSymbol(NSString *planet)
     if ( [planet isEqual:@"Juno"] || [planet isEqual:@"Hera"] )
         return @"Z";
     if ( [planet isEqual:@"Pallas"] || [planet isEqual:@"Minerva"] )
-        return @"(";
+        return @"[";
     if ( [planet isEqual:@"Vesta"] )
         return @"\\";
+    if ( [planet isEqual:@"Pholus"] )
+        return @"o";
     if ( [planet isEqual:@"AC"] )
         return @"P";
     if ( [planet isEqual:@"DC"] )
@@ -705,9 +897,11 @@ NSString *nodeSymbol(NSString *planet)
     if ( [planet isEqual:@"Juno"] || [planet isEqual:@"Hera"] )
         return @"ZT";
     if ( [planet isEqual:@"Pallas"] || [planet isEqual:@"Minerva"] )
-        return @"(T";
+        return @"[T";
     if ( [planet isEqual:@"Vesta"] )
         return @"\\T";
+    if ( [planet isEqual:@"Pholus"] )
+        return @"oT";
     return @"?";
 }
 
@@ -740,9 +934,11 @@ NSString *southNodeSymbol(NSString *planet)
     if ( [planet isEqual:@"Juno"] || [planet isEqual:@"Hera"] )
         return @"ZU";
     if ( [planet isEqual:@"Pallas"] || [planet isEqual:@"Minerva"] )
-        return @"(U";
+        return @"[U";
     if ( [planet isEqual:@"Vesta"] )
         return @"\\U";
+    if ( [planet isEqual:@"Pholus"] )
+        return @"oU";
     return @"?";
 }
 
@@ -811,9 +1007,19 @@ NSString *aspectSymbol(float deg, float orb)
     return nil;
 }
 
+/* bring deg to interval [0, 360]
+ */
+double mod360(double deg)
+{
+    while ( deg >= 360.0 )
+        deg -= 360.0;
+    while ( deg < 0.0 )
+        deg += 360.0;
+    return deg;
+}
 /* put angle within [0, 359.999]
  */
-double standardAngle( double angle )
+double standardAngle( double angle )	// same as angle
 {
     while ( angle >= 360.0 )
         angle -= 360.0;
@@ -821,6 +1027,16 @@ double standardAngle( double angle )
         angle += 360.0;
     return angle;
 }
+
+/* angle = object1 - object2
+ */
+double angle180( double deg1, double deg2 )
+{   double ang = angle(deg1, deg2);
+
+    if ( ang >= 180.0 )
+	ang = 360.0 - ang;
+    return ang;
+}
 /* angle = object1 - object2
  */
 double angle( double deg1, double deg2 )
@@ -865,3 +1081,32 @@ double harmony( double obj, double obj1, double obj2)
         return 180.0 * b / a;
     return -180.0 * (180.0 - b) / (360.0 - a);
 }
+
+
+/* check degFr <= deg <= degTo -> return YES
+ * we use a fixed modulus 360
+ * and pay attention to overflowing degrees at the 0/360 degree boundary
+ * parameter:
+ *   degFr  degree at earlier time
+ *   deg    degree to test
+ *   degTo  degree at later time
+ *   retro  NO  = direct     -> degFr < deg < degTo
+ *          YES = retrograde -> degTo < deg < degFr
+ */
+BOOL vhfIsDegreeInsideRange(double deg, double degFr, double degTo, BOOL retro)
+{
+    if (retro)	// exchange degFr and degTo
+    {   double  v = degFr;
+        degFr = degTo;
+        degTo = v;
+    }
+    if (degFr < 0.0)	// bring degFr above 0 (2006-05-31)
+        degFr += 360.0;
+    if (degTo < degFr)	// bring degTo behind degFr
+        degTo += 360.0;
+    if (deg < degFr)	// bring deg behind degFr
+        deg += 360.0;
+    if ( degFr <= deg && deg <= degTo)
+        return YES;
+    return NO;
+}
